//-*-c++-*------------------------------------------------------------
//
// File name : patPythonResults.h
// Author :    \URL[Michel Bierlaire]{http://roso.epfl.ch/mbi}
// Date :      Sat Jul 22 15:44:04 2006
//
//--------------------------------------------------------------------

#ifndef patPythonResults_h
#define patPythonResults_h

#include <map>
#include <vector>
#include "patString.h"

class patPythonResults {

 public:

  // Time when the results are created
  const char* getTimeStamp() ;
  patString timeStamp ;

  // Version of biogeme
  const char* getVersion() ;
  patString version ;

  // Model description as provided by the user in the .mod file
  const char* getDescription() ;
  patString description ;

  // Type of model 
  const char* getModel() ;
  patString model ;

  // Type of draws (Random / Halton / Hess-Train)
  const char* getDrawsType() ;
  patString drawsType ;

  int numberOfDraws ;

  int numberOfParameters ;

  int numberOfObservations ;

  int numberOfIndividuals;
  
  // Loglikelihood for the trivial model where all alternatives are
  // equally likely
  double nullLoglikelihood ;
  
  // Loglikelihood at the starting point of the estimation
  double initLoglikelihood ;

  double finalLoglikelihood ;  
  
  double likelihoodRatioTest ;

  double rhoSquare ;

  double rhoBarSquare ;

  double finalGradientNorm ;

  const char* getVarianceCovariance() ;
  patString varianceCovariance  ;

  int totalNumberOfParameters ;

  vector<patString> paramNames ;
  map<patString, unsigned int> paramIndex;
  vector<double> estimates ;
  vector<double> stdErr ;
  vector<double> tTest ;
  vector<double> pValue ;
  vector<double> stdErrRobust ;
  vector<double> tTestRobust ;
  vector<double> pValueRobust ;
  vector<int>   fixed ;
  vector<int>   distributed ;

  const char* getParamName(unsigned int index) ;
  double getEstimate(unsigned int index) ;
  double getEstimate(const char* name) ;
  double getStdErr(unsigned int index) ;
  double getStdErr(const char* name) ;
  double getTTest(unsigned int index) ;
  double getTTest(const char* name) ;
  double getPValue(unsigned int index) ;
  double getPValue(const char* name) ;
  double getStdErrRobust(unsigned int index) ;
  double getStdErrRobust(const char* name) ;
  double getTTestRobust(unsigned int index) ;
  double getTTestRobust(const char* name) ;

  double getPValueRobust(unsigned int index) ;
  double getPValueRobust(const char* name) ;
  // 1 if the parameter is fixed, 0  otherwise
  int   getFixed(unsigned int index) ;
  int   getFixed(const char* name) ;
  // 1 if the parameter has a discrete distribution, 0  otherwise
  int   getDistributed(unsigned int index) ;
  int   getDistributed(const char* name) ;

} ;

#endif
