//-*-c++-*------------------------------------------------------------
//
// File name : bioArithBinaryMinus.cc
// Author :    Michel Bierlaire and Mamy Fetiarison
// Date :      Tue Apr 28 20:28:16 2009
//
//--------------------------------------------------------------------

#include <sstream>

#include "patDisplay.h"
#include "patErrNullPointer.h"
#include "patErrMiscError.h"
#include "bioArithUnaryMinus.h"
#include "bioArithBinaryMinus.h"
#include "bioArithConstant.h"
#include "bioExpressionRepository.h"

bioArithBinaryMinus::bioArithBinaryMinus(bioExpressionRepository* rep,
					 patULong par,
					 patULong left, 
					 patULong right,
					 patError*& err) : 
  bioArithBinaryExpression(rep,par,left,right,err)
{
}

bioArithBinaryMinus::~bioArithBinaryMinus() {}

patString bioArithBinaryMinus::getOperatorName() const {
  return ("-") ;
}

patReal bioArithBinaryMinus::getValue(patBoolean prepareGradient, patError*& err) {
#ifndef BIO_NOCHECK
  if (leftChild == NULL || rightChild == NULL) {
    err = new patErrNullPointer("bioExpression") ;
    WARNING(err->describe()) ;
    return patReal();
  }
#endif  
  patReal left = leftChild->getValue(prepareGradient, err) ;
#ifndef BIO_NOCHECK
  if (err != NULL) {
    WARNING(err->describe()) ;
    return patReal();
  }
#endif
  patReal right = rightChild->getValue(prepareGradient,err) ;
#ifndef BIO_NOCHECK
  if (err != NULL) {
    WARNING(err->describe()) ;
    return patReal();
  }
#endif
  patReal result = left - right ;

  return result ;
}

bioExpression* bioArithBinaryMinus::getDerivative(patULong aLiteralId, 
						  patError*& err) const {

  if (leftChild == NULL || rightChild == NULL) {
    err = new patErrNullPointer("bioExpression") ;
    WARNING(err->describe()) ;
    return NULL;
  }
  
  bioExpression* leftResult = leftChild->getDerivative(aLiteralId,err) ; 
  if (err != NULL) {
    WARNING(err->describe()) ;
    return NULL;
  }
  bioExpression* rightResult =  rightChild->getDerivative(aLiteralId,err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return NULL;
  }

  bioExpression* result = new bioArithBinaryMinus(theRepository,
						  patBadId,
						  leftResult->getId(),
						  rightResult->getId(),
						  err) ;
  if (err != NULL) {
    WARNING(err->describe());
    return NULL ;
  }
  return result ;
}

bioArithBinaryMinus* bioArithBinaryMinus::getDeepCopy(bioExpressionRepository* rep,
						      patError*& err) const {
  bioExpression* leftClone(NULL) ;
  bioExpression* rightClone(NULL) ;
  if (leftChild != NULL) {
    leftClone = leftChild->getDeepCopy(rep,err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return NULL ;
    }
    if (leftClone == NULL) {
      err = new patErrNullPointer("bioExpression") ;
      WARNING(err->describe()) ;
      return NULL ;
    }
  }
  if (rightChild != NULL) {
    rightClone = rightChild->getDeepCopy(rep,err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return NULL ;
    }
    if (rightClone == NULL) {
      err = new patErrNullPointer("bioExpression") ;
      WARNING(err->describe()) ;
      return NULL ;
    }
  }

  bioArithBinaryMinus* theNode = 
    new bioArithBinaryMinus(rep,
			    patBadId,
			    leftClone->getId(),
			    rightClone->getId(),err) ;
  if (err != NULL) {
    WARNING(err->describe());
    return NULL ;
  }

  return theNode ;
}

bioArithBinaryMinus* bioArithBinaryMinus::getShallowCopy(bioExpressionRepository* rep,
						      patError*& err) const {
  bioArithBinaryMinus* theNode = 
    new bioArithBinaryMinus(rep,
			    patBadId,
			    leftChild->getId(),
			    rightChild->getId(),err) ;
  if (err != NULL) {
    WARNING(err->describe());
    return NULL ;
  }

  return theNode ;
}

patString bioArithBinaryMinus::getSimpleCppCode(patError*& err) {
  if (leftChild == NULL || rightChild == NULL) {
    err = new patErrNullPointer("bioExpression") ;
    WARNING(err->describe()) ;
    return patString();
  }
  
  patString vLeft = leftChild->getSimpleCppCode(err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return patString();
  }
  patString vRight = rightChild->getSimpleCppCode(err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return patString();
  }
  stringstream str ;
  str << "(" << vLeft << ")-(" << vRight << ")" ;
  return patString(str.str()) ;
}

patString bioArithBinaryMinus::getCppCode(patString prefix, patError*& err) {
  if (codeMustBeGenerated) {
  if (leftChild == NULL || rightChild == NULL) {
    err = new patErrNullPointer("bioExpression") ;
    WARNING(err->describe()) ;
    return patString();
  }
  
  patBoolean lSimple = leftChild->isSimple() ;
  patString vLeft ;
  if (lSimple) {
    vLeft = leftChild->getSimpleCppCode(err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return patString();
    }
  }
  else {
    vLeft = leftChild->getCppCode(prefix, err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return patString();
    }
  }
  patBoolean rSimple = rightChild->isSimple() ;
  patString vRight ;
  if (rSimple) {
    vRight = rightChild->getSimpleCppCode(err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return patString();
    }
  }
  else {
    vRight = rightChild->getCppCode(prefix, err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return patString();
    }
  }
  stringstream str ;
  if (!lSimple) {
    str << vLeft << endl ;
  }
  if (!rSimple) {
    str << vRight << endl ;
  }
  str << getCppResultDeclaration(prefix) << "= (" ;
  if (lSimple) {
    str << vLeft ;
  }
  else {
    str << leftChild->getCppResult(prefix) ;
  }
  str << " ) - ( " ;
  if (rSimple) {
    str << vRight ;
  }
  else {
    str << rightChild->getCppResult(prefix) ;
  }
  str << ");" << endl ;
  str << "    if (!finite(" << getCppResult(prefix) << ")) {" << endl ;
  str << "      " << getCppResult(prefix) << " =  patMaxReal ;" << endl ;
  str << "    }" << endl ;
  //  str << "DEBUG_MESSAGE(\"" << getCppResult(prefix) << "= \" << " << getCppResult(prefix) << ");" << endl ;
  codeMustBeGenerated = patFALSE ;
  return patString(str.str()) ;
  }
  else {
    return patString() ;
  }

}

patString bioArithBinaryMinus::getExpressionString() const {
  stringstream str ;
  str << '-' ;
  if (leftChild != NULL) {
    str << '{' << leftChild->getExpressionString() << '}' ;
  }
  if (rightChild != NULL) {
    str << '{' << rightChild->getExpressionString() << '}' ;
  }
  return patString(str.str()) ;

}



bioFunctionAndDerivatives* bioArithBinaryMinus::getNumericalFunctionAndGradient(vector<patULong> literalIds,
							   patError*& err) {
  if (result.empty()) {
    result.resize(literalIds.size()) ;
  }
  bioFunctionAndDerivatives* l = leftChild->getNumericalFunctionAndGradient(literalIds,err) ;
#ifndef BIO_NOCHECK
  if (err != NULL) {
    WARNING(err->describe()) ;
    return NULL ;
  }
#endif
  bioFunctionAndDerivatives* r = 
    rightChild->getNumericalFunctionAndGradient(literalIds,err) ;  
#ifndef BIO_NOCHECK
if (err != NULL) {
    WARNING(err->describe()) ;
    return NULL ;
  }
#endif
  result.theFunction =  l->theFunction - r->theFunction ;
  for (patULong i = 0 ; i < literalIds.size() ; ++i) {
    result.theGradient[i] = l->theGradient[i] - r->theGradient[i] ;
  }
  return &result ;
}
