//-*-c++-*------------------------------------------------------------
//
// File name : bioArithNormalPdf.cc
// Author :    Michel Bierlaire
// Date :      Mon Jun 27 18:22:36 2011
//
//--------------------------------------------------------------------

#include "patDisplay.h"
#include "bioArithNormalPdf.h"
#include "patErrNullPointer.h"
#include "patErrMiscError.h"

bioArithNormalPdf::bioArithNormalPdf(bioExpressionRepository* rep,
				     patULong par,
				     patULong left,
				     patError*& err) :
  bioArithUnaryExpression(rep,par,left,err), invSqrtTwoPi(0.3989422804) {

}

bioArithNormalPdf::~bioArithNormalPdf() {

}
patString bioArithNormalPdf::getOperatorName() const {
  return patString("normalPdf") ;
}

patReal bioArithNormalPdf::getValue(patBoolean prepareGradient, patError*& err)  {
#ifndef BIO_NOCHECK
  if (child == NULL) {
    err = new patErrNullPointer("bioExpression") ;
    WARNING(err->describe()) ;
    return patReal();
  }
#endif  
  patReal result = child->getValue(prepareGradient, err) ;
#ifndef BIO_NOCHECK
  if (err != NULL) {
    WARNING(err->describe()) ;
    return patReal();
  }
#endif

  return(invSqrtTwoPi * exp(-result * result / 2.0)) ;
}

bioExpression* bioArithNormalPdf::getDerivative(patULong aLiteralId, patError*& err) const {
  err = new patErrMiscError("Not yet implemented") ;
  WARNING(err->describe()) ;
  return NULL ;
}

bioArithNormalPdf* bioArithNormalPdf::getDeepCopy(bioExpressionRepository* rep,
						  patError*& err) const {

  bioExpression* leftClone(NULL) ;
  if (child != NULL) {
    leftClone = child->getDeepCopy(rep,err) ;
  }
  bioArithNormalPdf* theNode = 
    new bioArithNormalPdf(rep,patBadId,leftClone->getId(),err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return NULL ;
  }
  return theNode ;
}

bioArithNormalPdf* bioArithNormalPdf::getShallowCopy(bioExpressionRepository* rep,
						     patError*& err) const {

  bioArithNormalPdf* theNode = 
    new bioArithNormalPdf(rep,patBadId,child->getId(),err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return NULL ;
  }
  return theNode ;
}

patString bioArithNormalPdf::getCppCode(patString prefix, patError*& err) {
  err = new patErrMiscError("Not yet implemented") ;
  WARNING(err->describe()) ;
  return patString() ;
}

patString bioArithNormalPdf::getSimpleCppCode(patError*& err) {
  err = new patErrMiscError("Not yet implemented") ;
  WARNING(err->describe()) ;
  return patString() ;

}
patString bioArithNormalPdf::getExpressionString() const {
  stringstream str ;
  str << "$NPDF" ;
  if (child != NULL) {
    str << '{' << child->getExpressionString() << '}' ;
  }
  return patString(str.str()) ;
}

bioFunctionAndDerivatives* bioArithNormalPdf::getNumericalFunctionAndGradient(vector<patULong> literalIds, patError*& err) {

  if (result.empty()) {
    result.resize(literalIds.size()) ;
  }
  bioFunctionAndDerivatives* c = child->getNumericalFunctionAndGradient(literalIds,err) ;
#ifndef BIO_NOCHECK
  if (err != NULL) {
    WARNING(err->describe()) ;
    return NULL ;
  }
#endif
  result.theFunction =  invSqrtTwoPi * exp(- c->theFunction * c->theFunction / 2.0) ;
  for (patULong i = 0 ; i < literalIds.size() ; ++i) {
    if (c->theGradient[i] == 0.0) {
      result.theGradient[i] = 0.0 ;
    }
    else {
      result.theGradient[i] = - result.theFunction * c->theFunction * c->theGradient[i] ;
    }
  }
  return &result ;

}
