//-*-c++-*------------------------------------------------------------
//
// File name : bioModel.cc
// Author :    Michel Bierlaire and Mamy Fetiarison
// Date :      Thu May  7 13:36:48 WEST 2009
//
//--------------------------------------------------------------------

#include "bioModel.h"
#include "bioArithLiteral.h"
#include "patDisplay.h"
#include "bioExpressionRepository.h"

bioModel::bioModel(): theFormula(patBadId), 
		      statistics(NULL), 
		      constraints(NULL), 
		      simulation(NULL) ,
		      excludeExpr(patBadId),
		      theRepository(NULL) {
}

void bioModel::setRepository(bioExpressionRepository* rep) {
  theRepository = rep ;
}

void bioModel::setFormula(patULong f) {
  theFormula = f ;
}

patULong bioModel::getFormula() {
  return theFormula ;
}

void bioModel::setExcludeExpr(patULong e) {
  excludeExpr = e ;
}

patULong bioModel::getExcludeExpr() {
  return excludeExpr ;
}

void bioModel::setStatistics(map<patString, patULong>* stat) {
  statistics = stat ;
}
 
void bioModel::setConstraints(map<patString, patULong>* constr) {
  constraints = constr ;
}

void bioModel::setSimulation(bioArithPrint* simul) {
  simulation = simul ;
}

map<patString, patULong>* bioModel::getStatistics() {
  return statistics ;
}

map<patString, patULong>* bioModel::getConstraints() {
  return constraints ;
}

bioArithPrint* bioModel::getSimulation() {
  return simulation ;
}


patBoolean bioModel::mustEstimate() const {
  patError* err(NULL) ;
  if (theFormula == patBadId) {
    return patFALSE ;
  }
  bioExpression* f = theRepository->getExpression(theFormula) ;
  if (f == NULL) {
    return patFALSE ;
  }
  if (f->getExpression(err) == "0") {
    return patFALSE ;
  }
  if (err != NULL) {
    return patFALSE ;
  }
  return patTRUE  ;
}

patBoolean bioModel::mustSimulate() const {
  return (simulation != NULL) ;
}

bioExpressionRepository* bioModel::getRepository() {
  return theRepository ;
}
