//-*-c++-*------------------------------------------------------------
//
// File name : patZeroOneAntithetic.cc
// Author :    Michel Bierlaire
// Date :      Sun Jun 13 11:11:00 2010
//
//--------------------------------------------------------------------

#include "patZeroOneAntithetic.h"
#include "patUniform.h" 
#include "patDisplay.h"
#include "patErrMiscError.h"
#include "patFileNames.h"


patZeroOneAntithetic::patZeroOneAntithetic(patULong R,
					   patULong s,
					   patBoolean dumpDrawsOnFile,
					   patError*& err) : 
  patRandomNumberGenerator(dumpDrawsOnFile),
  uniformNumberGenerator(NULL),
  logFile(NULL) ,
  nbrOfDraws(R),
  nbrOfSeries(s),
  theDraws(s,vector<patReal>(ceil(R/2.0))),
  first(patTRUE),
  currentValue(0),
  currentSerie(0) {
  if (dumpDrawsOnFile) {
    logFile = new ofstream(patFileNames::the()->getUnifDrawLogFile(err).c_str()) ;  }
}

patZeroOneAntithetic::~patZeroOneAntithetic() {
  if (logFile != NULL) {
    logFile->close() ;
    DELETE_PTR(logFile) ;
  }
}


void patZeroOneAntithetic::setUniform(patUniform* rng) {
  uniformNumberGenerator = rng ;
  patError* err(NULL) ;
  for (patULong j = 0 ; j < nbrOfSeries ; ++j) {
    for (patULong i = 0 ; i < theDraws[j].size() ; ++i) {
      zeroOneDraw = uniformNumberGenerator->getUniform(err) ;
      if (err != NULL) {
	WARNING(err->describe()) ;
	return ;
      }
      theDraws[j][i] = zeroOneDraw ;
    }
  }


}

patReal patZeroOneAntithetic::getNextValue(patError*& err ) {

  if (currentValue == patBadId || currentSerie == patBadId) {
    generate(err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return patReal() ;
    }
  }

  patReal result ;
  if (first) {
    result = theDraws[currentSerie][currentValue] ;
  }
  else {
    result = 1.0-theDraws[currentSerie][currentValue] ;
  }

  ++currentValue ;
  if (currentValue == theDraws[currentSerie].size()) {
    if (first) {
      currentValue = 0 ;
      first = patFALSE;
    }
    else {
      // Next serie
      ++currentSerie ;
      if (currentSerie == nbrOfSeries) {
	currentSerie = patBadId ;
	currentValue = patBadId ;
      }
      else {
	currentValue = 0 ;
	first = patTRUE;
      }
    }
  }

  if (logFile != NULL) {
    *logFile << result << endl ;
  }
  return result ;
}

patReal patZeroOneAntithetic::getZeroOneDraw(patError*& err) {
  err = new patErrMiscError("Not implemented") ;
  WARNING(err->describe()) ;
  return patReal() ;
  
}

patBoolean patZeroOneAntithetic::isSymmetric() const {
  return patFALSE ;
}

void patZeroOneAntithetic::generate(patError*& err) {
  
  currentValue = 0 ;
  currentSerie = 0 ;
  for (patULong j = 0 ; j < nbrOfSeries ; ++j) {
    for (patULong i = 0 ; i < theDraws[j].size() ; ++i) {
      theDraws[j][i] = uniformNumberGenerator->getUniform(err) ;
      if (err != NULL) {
	WARNING(err->describe()) ;
	return ;
      }
    }
  }

}
