from biogeme import *
from mev import *

# Implements the nested logit model directly from the probability
# formula. It is the most efficient function, computationally
# speaking.
#
# Nests are defined as follows:
#nesta = MUA , [1,2,3]
#nestb = MUB , [4,5,6]
#nests = nesta, nestb
#
# Implements the nested logit model as a MEV model. 
def getMevForNested(V,availability,nests) :

    y = {}
    for i,v in V.items() :
        y[i] = exp(v)
    
    Gi = {}
    for m in nests:
        sum = 0.0
        for i in m[1]:
            sum += availability[i] * y[i] ** m[0]
        for i in m[1]:
            Gi[i] = availability[i] * y[i]**(m[0]-1.0) * sum ** (1.0/m[0] - 1.0)
    return Gi


# Implements the nested logit model as a MEV model. 
def nested(V,availability,nests,choice) :

    Gi = getMevForNested(V,availability,nests)
    P = mev(V,Gi,availability,choice) 
    return P

# Implements the nested logit model as a MEV model. mu is also a parameter here
def nestedMevMu(V,availability,nests,choice,mu) :

    chosen = Elem(V,choice)
    y = {}
    for i,v in V.items() :
        y[i] = exp(v-chosen)
    
    Gi = {}
    for m in nests:
        sum = 0.0
        for i in m[1]:
            sum += availability[i] * y[i] ** m[0]
        for i in m[1]:
            Gi[i] = availability[i] * mu * y[i]**(m[0]-1.0) * sum ** ((mu/m[0]) - 1.0)
    
    P = mev(V,Gi,availability,choice) 
    return P
